#! /bin/sh

### Firewall Rule Registering Script ##########################################
#                                                                             #
#  Usage : regfw.sh [-t Firewall_Type] [{-oip|-i} Outside_I/F_IP_Address] \   #
#          [-fip Flet's_Square_I/F_IP_Address ] [-q YES]                      #
#                                                                             #
###############################################################################


# =============================================================================
# Definition
# =============================================================================
#
# ----- define common variables and functions -----
. /usr/local/richscripts/etc/common.lib


# =============================================================================
# Start of "breakdown"
# =============================================================================
#
star_of_breakdown="YES"
while [ -n "$star_of_breakdown" ]; do
star_of_breakdown=""


# =============================================================================
# Preclusion ot multi-execution
# =============================================================================
#
counter=10
while [ $counter -gt 0 ]; do
  $PATH_LOCKING -n $FILE_REGFW -t 15
  if [ $? -eq 0 ]; then
    locking="YES"
    break
  else
    counter=`expr $counter - 1`
    $CMD_SLEEP 1
  fi
done
if [ $counter -le 0 ]; then
  debug_log "regfw.sh: the previous process is freezing"
  break
fi


# =============================================================================
# Check whether outside interface has a fixed IP address or not
# =============================================================================
#
[ -n "$oip" ] && flag_fixedip="YES"


# =============================================================================
# Register properties about firewall
# -----------------------------------------------------------------------------
#  0. oip detecting sub routine
#  1. about firewall program file
#  2. about outside I/F
#  3. about Flets's Square I/F (if available)
#  4. about the outside NTP server (if required)
# =============================================================================
#
# ----- 0. oip detecting sub routine -----
# arg   : $1 ... interface name
# result: 0=The specified interface has gotten an IP address.
#         1=The specified interface has not been able to get an IP address.
#         2=The specified interface name is invalid.
confirm_linkingup() {
  local PATTERN_IPV4_BW_MATCH=${PATTERN_IPV4#^}
  local check
  local result          # The result of this function (See the upper)
  local ifconfig_output # STDOUT from ifconfig command
  local inet_status     # 0:"inet" will be not found yet.
                        # 1:"inet" has been found now.
                        # 2:"inet" has been found a loop ago.
  result=1
  ifconfig_output=`$CMD_IFCONFIG $1 2> /dev/null`
  if [ $? -eq 0 ]; then
    inet_status=0
    for check in $ifconfig_output; do
      if [ $inet_status -le 1 ]; then
        if [ $inet_status -eq 0 ]; then
          if [ "${check#inet}" = "$check" ]; then
            continue
          fi
        fi
        check=`echo $check | $CMD_GREP -E $PATTERN_IPV4_BW_MATCH`
        if [ -n "$check" ]; then
          FUNC_RET=$check
          result=0
          break
        else
          inet_status=$(( $inet_status + 1 ))
        fi
      else
        result=1
        break
      fi
    done
  else
    result=2
  fi
  return $result
}
#
# ----- 1. 2. 3. about firewall command, option and outside, Flet's I/F -----
arg_switch=""
for arg in $*
do
  if [ "$arg_switch" = "" ]; then
    arg_switch="$arg"
  elif [ "$arg_switch" = "-t" ]; then
    arg_switch=""
    chosen_fw="$arg"
  elif [ \( "$arg_switch" = "-oip" \) -o \( "$arg_switch" = "-i" \) ]; then
    arg_switch=""
    oip="$arg"
  elif [ "$arg_switch" = "-fip" ]; then
    arg_switch=""
    fip="$arg"
  elif [ "$arg_switch" = "-q" ]; then
    arg_switch=""
    firewall_quiet="$arg"
  fi
done
if   [ "$oip" = "DETECT" ]; then
  confirm_linkingup $oif
  if [ $? ]; then
    oip=$FUNC_RET
  else
    oip=""
  fi
fi
if [ -z "$oip" ]; then
  if [ -f "$PATH_CURRENT_IP" ]; then
    for oip in `$CMD_CAT $PATH_CURRENT_IP`; do
      break
    done
  fi
fi
check=`echo $oip | $CMD_GREP -E $PATTERN_IPV4`
if [ -z "$check" ]; then
  echo "Firewall rule cannot be updated! (Specify the outside I/F's IP address)"
  debug_log "regfw.sh: no outside I/F's IP address specified"
  break
fi
if [ -z "$fip" ]; then
  if [ -f "$PATH_CURRENT_IP_FLETS" ]; then
    for check in `$CMD_CAT $PATH_CURRENT_IP_FLETS`; do
      break
    done
    check=`echo $check | $CMD_GREP -E $PATTERN_IPV4`
    if [ -n "$check" ]; then
      fip="$check"
    fi
  fi
fi
#
# ----- 4. about the outside NTP server -----
if [ -f "$PATH_NTP_CONF" ]; then
  ntpsv_line=`$CMD_GREP server $PATH_NTP_CONF`
  for word in $ntpsv_line
  do
    if [ "$word" != "server" ]; then
      ntpsv_ip=$word
      break
    fi
  done
fi


# =============================================================================
# Register firewall rule
# =============================================================================
#
case $chosen_fw in
  IPFW)
    if [ -f "$PATH_IPFWRULE" ]; then
      case $firewall_quiet in
        [Yy][Ee][Ss])
          IPFW="$CMD_IPFW -q"
          ;;
        *)
          IPFW="$CMD_IPFW"
          ;;
      esac
      $IPFW -f flush
      . $PATH_IPFWRULE
    fi
    ;;
  IPFILTER)
    if [ -f "$PATH_IPFILTERRULE" ]; then
      . $PATH_IPFILTERRULE
    fi
    ;;
  IPCHAINS)
    if [ -f "$PATH_IPCHAINSRULE" ]; then
      . $PATH_IPCHAINSRULE
    fi
    ;;
  IPTABLES)
    if [ -f "$PATH_IPTABLESRULE" ]; then
      . $PATH_IPTABLESRULE
    fi
    ;;
  *)
    echo "Specify firewall type from the following with \"-t\" option."
    echo " IPFW    : FreeBSD IPFW"
    echo " IPFILTER: IP Filter"
    echo " IPCHAINS: Linux ipchains"
    echo " IPTABLES: Linux iptables(Netfilter)"
    debug_log "regfw.sh: unknown firewall program"
    break
    ;;
esac


# =============================================================================
# End of "breakdown"
# =============================================================================
#
done


# =============================================================================
# Unlock if necessary
# =============================================================================
#
if [ "$locking" = "YES" ]; then
  $PATH_LOCKING -n $FILE_REGFW -u
fi
